@@ -2,6 +2,19 @@
 
 NIL_PTR <- methods::new("externalptr")
 
+assert_sparse_matrix <- function(spmat) {
+
+  if (inherits(spmat, "dgCMatrix")) {
+    FALSE
+  } else if (inherits(spmat, "dsCMatrix")) {
+    if (!identical(spmat@uplo, "U"))
+      stop2("Only upper-triangle symmetric matrices are implemented.")
+    TRUE
+  } else {
+    stop2("Only classes 'dgCMatrix' and 'dsCMatrix' are currently implemented.")
+  }
+}
+
 ################################################################################
 
 #' Class SFBM
@@ -68,7 +81,9 @@ SFBM_RC <- methods::setRefClass(
   ),
 
   methods = list(
-    initialize = function(spmat, backingfile, symmetric = FALSE) {
+    initialize = function(spmat, backingfile) {
+
+      symmetric <- assert_sparse_matrix(spmat)
 
       sbkfile <- path.expand(paste0(backingfile, ".sbk"))
       assert_noexist(sbkfile)
@@ -95,8 +110,7 @@ SFBM_RC <- methods::setRefClass(
 
     add_columns = function(spmat, offset_i) {
 
-      if (!inherits(spmat, "dgCMatrix") && !inherits(spmat, "dsCMatrix"))
-        stop2("Only classes 'dgCMatrix' and 'dsCMatrix' are currently implemented.")
+      symmetric <- assert_sparse_matrix(spmat)
 
       assert_one_int(offset_i)
       assert_pos(offset_i, strict = FALSE)
@@ -106,7 +120,7 @@ SFBM_RC <- methods::setRefClass(
 
       offset_p <- tail(.self$p, 1)
 
-      if (inherits(spmat, "dsCMatrix")) {
+      if (symmetric) {
         new_p <- write_indval_sym(sbkfile, spmat@p, spmat@i, spmat@x,
                                   offset_p, offset_i)
       } else {
@@ -140,30 +154,23 @@ SFBM_RC <- methods::setRefClass(
 #'   or dsCMatrix (symmetric sparse matrix of type 'double').
 #' @param backingfile Path to file where to store data. Extension `.sbk` is
 #'   automatically added.
+#' @param compact Whether to use a compact format? Default is `FALSE`.
+#'   This is useful when non-zero values in columns are contiguous (or almost).
 #'
 #' @return The new [SFBM][SFBM-class].
 #'
 #' @rdname SFBM-class
 #' @export
 #'
-#' @examples
-#' spmat <- Matrix::rsparsematrix(1000, 1000, 0.01)
-#' class(spmat)
-#' (X <- as_SFBM(spmat))
+#' @example examples/example-SFBM.R
 #'
-as_SFBM <- function(spmat, backingfile = tempfile()) {
+as_SFBM <- function(spmat, backingfile = tempfile(), compact = FALSE) {
 
-  if (inherits(spmat, "dgCMatrix")) {
-    sym <- FALSE
-  } else if (inherits(spmat, "dsCMatrix")) {
-    sym <- TRUE
-    if (!identical(spmat@uplo, "U"))
-      stop2("Only upper-triangle symmetric matrices are implemented.")
+  if (compact) {
+    new("SFBM_compact", spmat = spmat, backingfile = backingfile)
   } else {
-    stop2("Only classes 'dgCMatrix' and 'dsCMatrix' are currently implemented.")
+    new("SFBM",         spmat = spmat, backingfile = backingfile)
   }
-
-  new("SFBM", spmat = spmat, backingfile = backingfile, symmetric = sym)
 }
 
 ################################################################################
@@ -181,3 +188,100 @@ setMethod("dim",    signature(x = "SFBM"), function(x) c(x$nrow, x$ncol))
 setMethod("length", signature(x = "SFBM"), function(x) prod(dim(x)))
 
 ################################################################################
+
+#' Class SFBM_compact
+#'
+#' A reference class for storing and accessing sparse matrix-like data stored
+#' in files on disk, in a compact format (when non-zero values in columns are
+#' contiguous).
+#'
+#' @details
+#' It inherits the fields and methods from class [SFBM][SFBM-class].
+#'
+#' @importFrom bigassertr assert_exist assert_noexist assert_dir
+#' @importFrom bigassertr assert_class assert_pos assert_one_int stop2
+#' @importFrom methods new
+#' @importFrom utils head tail
+#'
+#' @exportClass SFBM_compact
+#'
+SFBM_compact_RC <- methods::setRefClass(
+
+  "SFBM_compact",
+
+  contains = "SFBM",
+
+  fields = list(
+    first_i = "integer",
+
+    #### Active bindings
+    address = function() {
+
+      if (identical(.self$extptr, NIL_PTR)) {
+        .self$extptr <- getXPtrSFBM_compact(
+          path    = .self$backingfile,
+          n       = .self$nrow,
+          m       = .self$ncol,
+          p       = .self$p,
+          first_i = .self$first_i
+        )
+      }
+
+      .self$extptr
+    }
+  ),
+
+  methods = list(
+    initialize = function(spmat, backingfile) {
+
+      symmetric <- assert_sparse_matrix(spmat)
+
+      sbkfile <- path.expand(paste0(backingfile, ".sbk"))
+      assert_noexist(sbkfile)
+      assert_dir(dirname(sbkfile))
+
+      res <- write_val_compact(sbkfile, spmat@p, spmat@i, spmat@x, 0, 0L,
+                               symmetric = symmetric)
+
+      .self$backingfile <- normalizePath(sbkfile)
+      .self$nrow        <- spmat@Dim[1]
+      .self$first_i     <- res[[1]]
+      .self$p           <- res[[2]]
+      .self$extptr      <- NIL_PTR
+
+      .self
+    },
+
+    add_columns = function(spmat, offset_i) {
+
+      symmetric <- assert_sparse_matrix(spmat)
+
+      assert_one_int(offset_i)
+      assert_pos(offset_i, strict = FALSE)
+
+      sbkfile <- .self$sbk
+      assert_exist(sbkfile)
+
+      offset_p <- tail(.self$p, 1)
+
+      res <- write_val_compact(sbkfile, spmat@p, spmat@i, spmat@x,
+                               offset_p, offset_i, symmetric = symmetric)
+
+      .self$nrow    <- max(.self$nrow, spmat@Dim[1] + offset_i)
+      .self$first_i <- c(.self$first_i, res[[1]])
+      .self$p       <- c(head(.self$p, -1), res[[2]])
+      .self$extptr  <- NIL_PTR
+
+      .self
+    },
+
+    show = function() {  # TODO: add type when implementing 'float'
+      cat(sprintf(
+        "A compact Sparse Filebacked Big Matrix with %s rows and %s columns.\n",
+        .self$nrow, .self$ncol))
+      invisible(.self)
+    }
+  )
+)
+
+################################################################################
